package com.iamberry.wechat.tools;
import com.google.zxing.BarcodeFormat;
import com.google.zxing.EncodeHintType;
import com.google.zxing.MultiFormatWriter;
import com.google.zxing.common.BitMatrix;

import javax.imageio.ImageIO;
import java.io.File;
import java.io.OutputStream;
import java.io.IOException;
import java.awt.image.BufferedImage;
import java.util.Hashtable;

/**
 * Created by Administrator on 2017/8/9.
 */
public class QrCodeUtil {

   private static final int BLACK = 0xFF000000;
   private static final int WHITE = 0xFFFFFFFF;
   public static BufferedImage toBufferedImage(BitMatrix matrix) {
        int width = matrix.getWidth();
        int height = matrix.getHeight();
        BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        for (int x = 0; x < width; x++) {
            for (int y = 0; y < height; y++) {
                image.setRGB(x, y, matrix.get(x, y) ? BLACK : WHITE);
            }
        }
        return image;
   }
    public static void writeToFile(BitMatrix matrix, String format, File file) throws IOException {
        BufferedImage image = toBufferedImage(matrix);
        if (!ImageIO.write(image, format, file)) {
            throw new IOException("Could not write an image of format " + format + " to " + file);
        }
    }
    public static void writeToStream(BitMatrix matrix, String format, OutputStream stream) throws IOException {
        BufferedImage image = toBufferedImage(matrix);
        if (!ImageIO.write(image, format, stream)) {
        throw new IOException("Could not write an image of format " + format);
        }
    }

    /**
     * 创建个人二维码
     * @param url   扫描二维码所访问的链接
     * @param path  生成二维码路径
     * @throws Exception
     */
    public static String createQrCode(String url,String path) throws Exception {
        int width = 500;
        int height = 500;
        //二维码的图片格式
        String format = "png";
        //二维码保存路径
        StringBuilder str = new StringBuilder(path);
        str.append(format);

        File outputFile = new File(str.toString());
        //判断该二维码是否存在，如果存在，则不生成
        if (outputFile.exists()) {
            return str.toString();
        }
        Hashtable hints = new Hashtable();
        //内容所使用编码
        hints.put(EncodeHintType.CHARACTER_SET, "utf-8");
        BitMatrix bitMatrix = new MultiFormatWriter().encode(url,
        BarcodeFormat.QR_CODE, width, height, hints);
        //生成二维码
        writeToFile(bitMatrix, format, outputFile);
        return str.toString();
    }
}
